/************************************************************************************************

 Author        : ADI - Apps            www.analog.com/AduC7060

 Date          : November 2007

 File          : SPI_Master.c

 Hardware      : ADuC706x

 Description   : This SPI Loopback test.
				

*************************************************************************************************/
// Bit Definitions
#define BIT0  0x01
#define BIT1  0x02
#define BIT2  0x04
#define BIT3  0x08
#define BIT4  0x10
#define BIT5  0x20
#define BIT6  0x40
#define BIT7  0x80
#define BIT8  0x100
#define BIT9  0x200
#define BIT10 0x400
#define BIT11 0x800
#define BIT12 0x1000
#define BIT13 0x2000
#define BIT14 0x4000
#define BIT15 0x8000
#define BIT16 0x10000
#define BIT17 0x20000
#define BIT25 0x2000000

#include<aduc7060.h>
// Function Definitions
unsigned char szTxData[] = {0x1, 0x2, 0x10, 0x20, 0x40, 0x80};	// Array to send to Slave
unsigned char ucTxCount = 0;  // Array index variable for szTxData[]
unsigned char szRxData[6];	  // Array for reading Data from Slave
unsigned char ucRxCount = 0;  // Array index variable for szRxData[]
unsigned char ucWaitingForXIRQ0 = 1; // Flag to begin reading from Slave - controlled by XIRQ0
unsigned char ucDummyRd = 0;
unsigned char Overflow = 0;
int main(void)
{
    POWKEY1 = 0x1;
	POWCON0 = 0x78;		   // Set core to max CPU speed of 10.24Mhz
	POWKEY2 = 0xF4;

//	GP0DAT = BIT25;
	// Configure P0.0, P0.1,P0.2 and P0.3 for SPI mode
	GP0CON0 = BIT0 + BIT4 + BIT8 + BIT12; // Select SPI/I2C alternative function for P0[0...3]
	GP0KEY1 = 0x7;		   //Write to GP0KEY1
	GP0CON1	= 0x00;	   // Select SPI functionality for P0.0 to  P0.3
	GP0KEY2 = 0x13;		   //Write to GP0KEY2

	
	SPICON = BIT0 + BIT1   // Enable SPI + Enable SPI Master
		   + BIT6   // Clk pulses high at start of each bit + Initiate transfer on write to Tx FIFO
		    + BIT10  //MISO output + Loopback mode enabled  
		    + BIT7
		   + BIT11;		   //Continuous transfer mode + Enable IRQ on 1 byte transfer
	//	   + BIT14;
	SPICON |= BIT13; // Flush Tx FIFO
	SPICON &= ~BIT13; 
	SPIDIV  = 0x3F;	   		// Select 101kHz clock rate
	IRQCONE = BIT1;			// Rising edge of XIRQ0
	IRQEN = BIT12 + BIT13; 		// Enable SPI + XIRQ0 interrupt

	// Begin Master Transmit sequence
	ucTxCount = 0;
	SPITX = szTxData[ucTxCount++];
	
	ucWaitingForXIRQ0 = 1;
	ucRxCount = 0;
		while (1)
	{
	 	
		if (ucWaitingForXIRQ0 == 0)	// Wait for XIRQ0 to trigger - cleared in XIRQ0 ISR
		{
			ucWaitingForXIRQ0 = 1;
			ucRxCount = 0;
			SPICON |= BIT12; // Flush Rx FIFO
			SPICON &= ~BIT12; 
			SPICON &= ~BIT6; // Generate Interrupt on receiving data
			ucDummyRd = SPIRX;
		}
	}
}

void IRQ_Handler(void) __irq
{
	unsigned long IRQSTATUS = 0;
	unsigned int SPIMSTATUS = 0;
	unsigned char Underflow = 0;

	IRQSTATUS = IRQSTA;	   // Read off IRQSTA register
	if ((IRQSTATUS & BIT13) == BIT13)	//XIRQ0 interrupt source
	{
		ucWaitingForXIRQ0 = 0;			// Enable Read request from Slave.
		IRQCLRE = BIT13;
	}

	if ((IRQSTATUS & BIT12) == BIT12)	//If SPI Master interrupt source
	{
	   SPIMSTATUS = SPISTA;
	   if ((SPIMSTATUS & BIT4) == BIT4) // SPI Tx underflow
	   {
			SPITX = szTxData[ucTxCount++];
	   }
	   if ((SPIMSTATUS & BIT5) == BIT5) // If SPI Master Tx IRQ
	   {
			if (ucTxCount < 6)		   // Have max 6 bytes been sent?
				SPITX = szTxData[ucTxCount++];	 // Load Tx buffer
	   }
	   if ((SPIMSTATUS & BIT6) == BIT6) // If SPI Master Rx IRQ
	   {
			if (ucRxCount < 6)		   // Have max 6 bytes been received?
			{
				szRxData[ucRxCount] = SPIRX;  // Read Rx buffer
				SPITX = szTxData[ucTxCount++];

				ucRxCount++;
			}
			else
				ucRxCount = 0;
	   }
	   if ((SPIMSTATUS & BIT8) == BIT8) // Rx FIFO overflow
	   {
			Overflow++;
	   }
	}
}


